import { Vector2 } from "@skeldjs/util";
import { SkeldjsClient } from "@skeldjs/client";
import { CustomNetworkTransform, PlayerData, PlayerDataResolvable, GameMap } from "@skeldjs/core";
import { EventEmitter, ExtractEventTypes } from "@skeldjs/events";
import { PathfinderConfig } from "./interface/PathfinderConfig";
import { Grid } from "./util/Grid";
import { Node } from "./util/Node";
import { PathfinderEndEvent, EngineMoveEvent, PathfinderPauseEvent, EngineRecalculateEvent, PathfinderStartEvent, PathfinderStopEvent } from "./events";
export declare type SkeldjsPathfinderEvents = ExtractEventTypes<[
    PathfinderEndEvent,
    EngineMoveEvent,
    PathfinderPauseEvent,
    EngineRecalculateEvent,
    PathfinderStartEvent,
    PathfinderStopEvent
]>;
/**
 * Represents a pathfinding utility for the {@link SkeldjsClient SkeldJS Client}.
 *
 * See {@link SkeldjsPathfinderEvents} for events to listen to.
 */
export declare class SkeldjsPathfinder extends EventEmitter<SkeldjsPathfinderEvents> {
    private client;
    config: PathfinderConfig;
    private _tick;
    private _moved;
    isPaused: boolean;
    /**
     * The destination of the pathfinder.
     */
    destination?: Vector2;
    /**
     * The grid of nodes for the pathfinder engine.
     */
    grid?: Grid;
    /**
     * The current intended path of the pathfinder.
     */
    path?: Node[];
    /**
     * The player that the pathfinder is currently finding.
     */
    following?: PlayerData;
    constructor(client: SkeldjsClient, config?: PathfinderConfig);
    private get snode();
    private get dnode();
    get position(): Vector2 | undefined;
    get transform(): CustomNetworkTransform | undefined;
    get myPlayer(): PlayerData<SkeldjsClient> | undefined;
    getMapId(): -1 | GameMap;
    private getVentForMap;
    private _ontick;
    recalculate(): Promise<void>;
    pause(): void;
    start(): void;
    private _stop;
    stop(): void;
    private _go;
    go(pos: PlayerDataResolvable | Vector2 | Node): void;
    vent(ventid: number): void;
    private _handleMove;
    private _handleLeave;
    follow(player: PlayerDataResolvable): void;
    static MovementInterval: 6;
}
