"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getShortestPath = void 0;
const heap_1 = __importDefault(require("heap"));
const heuristic = (a, b) => Math.hypot(a, b);
function getShortestPath(grid, start, end) {
    const heap = new heap_1.default((a, b) => {
        return a.f - b.f;
    });
    start.g = 0;
    start.h = heuristic(start.x - end.x, start.y - end.y);
    start.parent = undefined;
    heap.push(start);
    grid.pathid++;
    start.opened = false;
    let closest = start;
    while (heap.size() > 0) {
        const current = heap.pop();
        current.closed = true;
        grid.dirty.add(current);
        if (!(closest === null || closest === void 0 ? void 0 : closest.h) || !current.h || current.h < closest.h) {
            closest = current;
        }
        if (current === end) {
            const path = current.getPath();
            path.shift();
            return path;
        }
        const neighbors = current.getNeighbors().filter(node => !node.blocked);
        for (const neighbor of neighbors) {
            if (neighbor.closed)
                continue;
            const g = (current.g +
                (neighbor.x - current.x === 0 ||
                    neighbor.y - current.y === 0
                    ? 1
                    : Math.SQRT2)) *
                neighbor.weight;
            if (!neighbor.closed || g < neighbor.g) {
                neighbor.g = g;
                neighbor.h || (neighbor.h = 1 * heuristic(neighbor.x - end.x, neighbor.y - end.y));
                neighbor.parent = current;
                if (!neighbor.opened) {
                    heap.push(neighbor);
                    neighbor.opened = true;
                    grid.dirty.add(neighbor);
                }
                else {
                    heap.updateItem(neighbor);
                }
            }
        }
    }
    if (closest) {
        const path = closest.getPath();
        path.shift();
        return path;
    }
    return [];
}
exports.getShortestPath = getShortestPath;
//# sourceMappingURL=engine.js.map