"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Grid = void 0;
const util_1 = require("@skeldjs/util");
const Node_1 = require("./Node");
class Grid {
    constructor(nodes, basex, basey, width, height, density) {
        this.basex = basex;
        this.basey = basey;
        this.width = width;
        this.height = height;
        this.density = density;
        this.nodes = nodes;
        this.dirty = new Set;
        this.pathid = 0;
    }
    static fromBuffer(buffer) {
        const reader = util_1.HazelReader.from(buffer);
        const basex = reader.int16();
        const basey = reader.int16();
        const width = reader.uint16();
        const height = reader.uint16();
        const density = reader.uint8();
        const actual_width = width * density;
        const actual_height = height * density;
        const grid = new Grid(new Array(actual_height)
            .fill(null)
            .map(() => new Array(actual_width).fill(null)), basex, basey, width, height, density);
        let i = 0;
        while (reader.left) {
            const num = reader.upacked();
            const blocked = reader.bool();
            const weight = reader.float();
            for (let j = 0; j < num; j++) {
                const x = i % actual_height;
                const y = ~~(i / actual_height);
                if (!grid.nodes[y]) {
                    grid.nodes.push([]);
                }
                grid.nodes[y][x] = new Node_1.Node(grid, x, y, blocked, weight);
                i++;
            }
        }
        return grid;
    }
    static create(basex, basey, width, height, density) {
        const actual_width = width * density;
        const actual_height = height * density;
        const grid = new Grid([], basex, basey, width, height, density);
        grid.nodes = new Array(actual_height).fill(null).map((_, y) => {
            return new Array(actual_width).fill(null).map((_, x) => {
                return new Node_1.Node(grid, x, y, false, 1);
            });
        });
        return grid;
    }
    get actual_width() {
        return this.width * this.density;
    }
    get actual_height() {
        return this.height * this.density;
    }
    nearest(x, y) {
        const nodex = ~~((x - this.basex) * this.density);
        const nodey = ~~((y - this.basey) * this.density);
        return this.nodes[nodey][nodex];
    }
    actual(x, y) {
        return new util_1.Vector2(x / this.density + this.basex, y / this.density + this.basey);
    }
    at(i) {
        const x = ~~(i / this.actual_height);
        const y = i % this.actual_height;
        return this.get(x, y);
    }
    get(x, y) {
        var _a, _b;
        return (_b = (_a = this.nodes) === null || _a === void 0 ? void 0 : _a[y]) === null || _b === void 0 ? void 0 : _b[x];
    }
    set(x, y) {
        if (!this.get(x, y))
            return;
        this.nodes[y][x].blocked = true;
    }
    unset(x, y) {
        if (!this.get(x, y))
            return;
        this.nodes[y][x].blocked = false;
    }
    reset() {
        for (const node of this.dirty) {
            node.opened = false;
            node.closed = false;
            node.g = undefined;
            node.h = undefined;
        }
    }
    createBuffer() {
        const actual_width = this.width * this.density;
        const actual_height = this.height * this.density;
        const writer = util_1.HazelWriter.alloc(actual_width * actual_height);
        writer.int16(this.basex);
        writer.int16(this.basey);
        writer.uint16(this.width);
        writer.uint16(this.height);
        writer.uint8(this.density);
        let num = 0;
        let current = undefined;
        for (let y = 0; y < this.actual_height; y++) {
            for (let x = 0; x < this.actual_width; x++) {
                if (!current) {
                    current = this.nodes[y][x];
                }
                if (current.blocked !== this.nodes[y][x].blocked ||
                    current.weight !== this.nodes[y][x].weight) {
                    writer.upacked(num);
                    writer.bool(current.blocked);
                    writer.float(current.weight);
                    current = this.nodes[y][x];
                    num = 0;
                }
                num++;
            }
        }
        writer.upacked(num);
        writer.bool((current === null || current === void 0 ? void 0 : current.blocked) || false);
        writer.float((current === null || current === void 0 ? void 0 : current.weight) || 0);
        writer.realloc(writer.cursor);
        return writer.buffer;
    }
}
exports.Grid = Grid;
//# sourceMappingURL=Grid.js.map